unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Spin, ComCtrls, ToolWin,
  Menus, ImgList, ActnList, ExtCtrls, FWEventLog;

type
  TEventLogDemoMainForm = class(TForm)
    lvEvents: TListView;
    StatusBar: TStatusBar;
    ToolBar: TToolBar;
    tbRegisterAsSource: TToolButton;
    ImageList: TImageList;
    tbDeRegister: TToolButton;
    tbSeparator1: TToolButton;
    pmQuery: TPopupMenu;
    N1: TMenuItem;
    N2: TMenuItem;
    N3: TMenuItem;
    tbSeparator2: TToolButton;
    tbAddNotification: TToolButton;
    tb: TToolButton;
    tbClear: TToolButton;
    tbSeparator3: TToolButton;
    tbSeparator4: TToolButton;
    tbCreateBackup: TToolButton;
    tbOpenBackup: TToolButton;
    tbNewRecord: TToolButton;
    MainMenu: TMainMenu;
    N4: TMenuItem;
    N5: TMenuItem;
    N6: TMenuItem;
    N7: TMenuItem;
    N8: TMenuItem;
    N9: TMenuItem;
    N10: TMenuItem;
    N11: TMenuItem;
    N12: TMenuItem;
    N13: TMenuItem;
    N14: TMenuItem;
    N15: TMenuItem;
    N16: TMenuItem;
    N17: TMenuItem;
    N18: TMenuItem;
    N19: TMenuItem;
    N20: TMenuItem;
    ToolButton1: TToolButton;
    CoolBar: TCoolBar;
    ActionList: TActionList;
    acRegister: TAction;
    acDeregister: TAction;
    acMakeBackup: TAction;
    acOpenBackup: TAction;
    acRecordsCount: TAction;
    acRecordsCountWithOldest: TAction;
    acIsLogFull: TAction;
    acAddNotification: TAction;
    acRemoveNotification: TAction;
    acAddRecord: TAction;
    acClearLog: TAction;
    tvEventTypes: TTreeView;
    BackupSaveDialog: TSaveDialog;
    acOpen: TAction;
    acClose: TAction;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    N21: TMenuItem;
    N22: TMenuItem;
    N23: TMenuItem;
    N24: TMenuItem;
    Splitter: TSplitter;
    lvNotify: TListView;
    ImageListEvents: TImageList;
    gbNotify: TGroupBox;
    procedure Edit1KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure acRemoveNotificationExecute(Sender: TObject);
    procedure acAddNotificationExecute(Sender: TObject);
    procedure acRemoveNotificationUpdate(Sender: TObject);
    procedure acAddNotificationUpdate(Sender: TObject);
    procedure acAddRecordExecute(Sender: TObject);
    procedure acClearLogExecute(Sender: TObject);
    procedure tvEventTypesClick(Sender: TObject);
    procedure lvEventsCustomDraw(Sender: TCustomListView; const ARect: TRect;
      var DefaultDraw: Boolean);
    procedure tvEventTypesCollapsing(Sender: TObject; Node: TTreeNode;
      var AllowCollapse: Boolean);
    procedure acIsLogFullExecute(Sender: TObject);
    procedure acRecordsCountUpdate(Sender: TObject);
    procedure acCloseExecute(Sender: TObject);
    procedure acOpenExecute(Sender: TObject);
    procedure acCloseUpdate(Sender: TObject);
    procedure acOpenUpdate(Sender: TObject);
    procedure acRecordsCountExecute(Sender: TObject);
    procedure acRecordsCountWithOldestExecute(Sender: TObject);
    procedure acOpenBackupExecute(Sender: TObject);
    procedure acMakeBackupExecute(Sender: TObject);
    procedure acDeregisterExecute(Sender: TObject);
    procedure acRegisterExecute(Sender: TObject);
    procedure acDeregisterUpdate(Sender: TObject);
    procedure acRegisterUpdate(Sender: TObject);
    procedure lvEventsDblClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    Registered: Boolean;
    CurrData: Integer;
    BlockRedrawRecursion: Boolean;
    procedure VerifyRegistration;
    procedure OnEventRead(Sender: TObject;
      EventRecord: TFWEventLogRecord);
  public
    EventLog: TFWEventLog;
    RussianLayout: HKL;
    BackupShowing: Boolean;
    BackupFilePath: String;
    procedure RearrangelistView(LV: TListView);
  end;

var
  EventLogDemoMainForm: TEventLogDemoMainForm;

implementation

{$R *.dfm}

uses
  uProgress, uViewRecordData, uNewRecord, CustomOpenDialog;

procedure TEventLogDemoMainForm.acAddNotificationExecute(Sender: TObject);
begin
  if not EventLog.RegisterChangeNotify(OnEventRead) then
    ShowMessage(' ')
  else
   gbNotify.Visible := True;
end;

procedure TEventLogDemoMainForm.acAddNotificationUpdate(Sender: TObject);
begin
  acAddNotification.Enabled := (osRead in EventLog.OpenState) and
    not (osNotify in EventLog.OpenState);
end;

procedure TEventLogDemoMainForm.acAddRecordExecute(Sender: TObject);
begin
  dlgNewRec := TdlgNewRec.Create(nil);
  try
    dlgNewRec.ShowModal;
  finally
    dlgNewRec.Release;
  end;
end;

procedure TEventLogDemoMainForm.acClearLogExecute(Sender: TObject);
var
  SourceName, BackUpPath: String;
begin
  SourceName := RegEventSources[EventLog.ReadSourceType];
  BackUpPath := ExtractFilePath(ParamStr(0)) + 'ClearBackup\';
  ForceDirectories(BackUpPath);
  EventLog.Clear(BackUpPath + SourceName + '.evt');
  acOpenExecute(nil);
end;

procedure TEventLogDemoMainForm.acCloseExecute(Sender: TObject);
begin
  EventLog.Close(osRead);
end;

procedure TEventLogDemoMainForm.acCloseUpdate(Sender: TObject);
begin
  TAction(Sender).Enabled := osRead in EventLog.OpenState;
end;

procedure TEventLogDemoMainForm.acDeregisterExecute(Sender: TObject);
begin
  if EventLog.DeregisterApp then
    ShowMessage('    ')
  else
    ShowMessage('    ');
  VerifyRegistration;
end;

procedure TEventLogDemoMainForm.acDeregisterUpdate(Sender: TObject);
begin
  acDeregister.Enabled := Registered;
end;

procedure TEventLogDemoMainForm.acIsLogFullExecute(Sender: TObject);
begin
  if EventLog.IsEventLogFull then
    ShowMessage('   .')
  else
    ShowMessage('     .');
end;

procedure TEventLogDemoMainForm.acMakeBackupExecute(Sender: TObject);
begin
  if BackupSaveDialog.Execute then
    if EventLog.CreateBackup(BackupSaveDialog.FileName) then
      ShowMessage('    ')
    else
      ShowMessage('   ');
end;

procedure TEventLogDemoMainForm.acOpenBackupExecute(Sender: TObject);
begin
  EventLogOpenDialog := TEventLogOpenDialog.Create;
  try
    if not EventLogOpenDialog.Execute(Handle) then Exit;
    lvEvents.Items.BeginUpdate;
    try
      lvEvents.Items.Clear;
      if osBackUp in EventLog.OpenState then
        EventLog.Close(osBackUp);

      if EventLog.BackupOpen(EventLogOpenDialog.FileName,
        EventLogOpenDialog.EventSourceType) then
      begin
        dlgProgress := TdlgProgress.Create(nil);
        try
          dlgProgress.LoadBackupLog(EventLogOpenDialog.FileName);
        finally
          dlgProgress.Release;
          BackupShowing := True;
          if tvEventTypes.Items.Count = 5 then
          begin
            with tvEventTypes.Items.AddChild(tvEventTypes.Items.Item[4],
              EventLogOpenDialog.DisplayName)do
            begin
              StateIndex := 12;
              ImageIndex := 5;
            end;
            tvEventTypes.Items[4].Expand(True);
          end
          else
            tvEventTypes.Items[5].Text := EventLogOpenDialog.DisplayName;
          BackupFilePath := EventLogOpenDialog.FileName;
          Caption := '  [' + EventLogOpenDialog.FileName + ']';
        end;
      end
      else
        ShowMessage('   .');
    finally
      lvEvents.Items.EndUpdate;
    end;
  finally
    EventLogOpenDialog.Free;
  end;                      
end;

procedure TEventLogDemoMainForm.acOpenExecute(Sender: TObject);
var
  SourceName: String;
begin
  SourceName := '';
  if Sender <> nil then
    CurrData := TComponent(Sender).Tag;
  BackupShowing := False;
  if not (CurrData in [1..3]) then
  begin
    lvEvents.Items.Clear;
    Exit;
  end;
  SourceName := RegEventSources[TFWLocalEventSources(CurrData)];
  Caption := '  [' + SourceName + ']';
  if (osRead in EventLog.OpenState) then
    EventLog.Close(osRead);  
  if not EventLog.Open(TFWLocalEventSources(CurrData), osRead) then
    ShowMessage('   .');
  lvEvents.Items.BeginUpdate;
  try
    lvEvents.Items.Clear;
    dlgProgress := TdlgProgress.Create(nil);
    try
      dlgProgress.LoadEventLog('FWEventLogDemo');
    finally
      dlgProgress.Release;
    end;
  finally
    lvEvents.Items.EndUpdate;
  end;
end;

procedure TEventLogDemoMainForm.acOpenUpdate(Sender: TObject);
begin
  acOpen.Enabled := not (osRead in EventLog.OpenState);
end;

procedure TEventLogDemoMainForm.acRecordsCountExecute(Sender: TObject);
var
  RecordsCount: Integer;
begin
  RecordsCount := EventLog.RecordCount(BackupShowing, False);
  ShowMessage(Format('      %d',
    [RecordsCount]));
end;

procedure TEventLogDemoMainForm.acRecordsCountUpdate(Sender: TObject);
begin
  TAction(Sender).Enabled :=
     (osRead in EventLog.OpenState) or (osBackUp in EventLog.OpenState);
end;

procedure TEventLogDemoMainForm.acRecordsCountWithOldestExecute(
  Sender: TObject);
var
  RecordsCount: Integer;
begin
  RecordsCount := EventLog.RecordCount(BackupShowing, True);
  ShowMessage(Format('       %d',
    [RecordsCount]));
end;

procedure TEventLogDemoMainForm.acRegisterExecute(Sender: TObject);
begin
  if EventLog.RegisterApp then
    ShowMessage('     ')
  else
    ShowMessage('     ');
  VerifyRegistration;
end;

procedure TEventLogDemoMainForm.acRegisterUpdate(Sender: TObject);
begin
  acRegister.Enabled := not Registered;
end;       

procedure TEventLogDemoMainForm.acRemoveNotificationExecute(Sender: TObject);
begin
  EventLog.DeRegisterChangeNotify;
  gbNotify.Visible := False;
end;

procedure TEventLogDemoMainForm.acRemoveNotificationUpdate(Sender: TObject);
begin
  acRemoveNotification.Enabled := (osRead in EventLog.OpenState) and
    (osNotify in EventLog.OpenState);
end;

procedure TEventLogDemoMainForm.Edit1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  ShowMessage(IntToStr(Key));
end;

procedure TEventLogDemoMainForm.FormCreate(Sender: TObject);
var
  I: Integer;
begin
  RussianLayout := LoadKeyboardLayout('00000419', 0);
  EventLog := TFWEventLog.Create('FWEventLogDemo');
  VerifyRegistration;              
  tvEventTypes.Items[0].Expand(True);
  lvEvents.DoubleBuffered := True;
  for I := 0 to ActionList.ActionCount - 1 do
    ActionList.Actions[I].Update;
  BlockRedrawRecursion := False;
end;

procedure TEventLogDemoMainForm.FormDestroy(Sender: TObject);
begin
  EventLog.Free;
end;

procedure TEventLogDemoMainForm.lvEventsCustomDraw(Sender: TCustomListView;
  const ARect: TRect; var DefaultDraw: Boolean);
const
  LVNOITEMS_STR = '      .';
var
  TextLeft: Integer;
begin
  if TListView(Sender).Items.Count = 0 then
  begin
    DefaultDraw := False;
    if BlockRedrawRecursion then
    begin
      BlockRedrawRecursion := False;        
      TListView(Sender).Invalidate;
      Exit;
    end;
    BlockRedrawRecursion := True;
    TextLeft := (TListView(Sender).Width -
      TListView(Sender).Canvas.TextWidth(LVNOITEMS_STR)) div 2;
    TListView(Sender).Canvas.TextOut(TextLeft, 35, LVNOITEMS_STR);
  end;
end;

procedure TEventLogDemoMainForm.lvEventsDblClick(Sender: TObject);
var
  Rec: TFWEventLogRecord;
begin
  if lvEvents.Selected = nil then Exit;
  try
    if not EventLog.Read(BackupShowing,
      Integer(lvEvents.Selected.Data), Rec) then Abort;
  except
    ShowMessage('  ');
    Exit;
  end;

  dlgViewRecordData := TdlgViewRecordData.Create(nil);
  try
    dlgViewRecordData.ShowRecData(Rec);
    dlgViewRecordData.ShowModal;
  finally
    dlgViewRecordData.Release;
  end;         
end;

procedure TEventLogDemoMainForm.OnEventRead(Sender: TObject;
  EventRecord: TFWEventLogRecord);
begin
  with lvNotify.Items.Add do
  begin
    Data := Pointer(EventRecord.Number);
    Caption := TFWEventLog.EventTypeToString(EventRecord.EventType);
    StateIndex := Integer(EventRecord.EventType);
    SubItems.Add(DateToStr(EventRecord.TimeWritten));
    SubItems.Add(TimeToStr(EventRecord.TimeWritten));
    SubItems.Add(EventRecord.SourceName);
    SubItems.Add(EventRecord.Category);
    SubItems.Add(IntToStr(EventRecord.EventID));
    SubItems.Add(EventRecord.Account);
    SubItems.Add(EventRecord.ComputerName);
  end;
  RearrangelistView(lvNotify);
end;

procedure TEventLogDemoMainForm.RearrangelistView(LV: TListView);
const
  Plus = 107;
begin
  LV.SetFocus;
  keybd_event(VK_CONTROL, MapVirtualKey(VK_CONTROL, 0), 0, 0);
  keybd_event(Plus, MapvirtualKey(Plus, 0), KEYEVENTF_EXTENDEDKEY, 0);
  keybd_event(Plus, MapvirtualKey(Plus, 0),
    KEYEVENTF_EXTENDEDKEY or KEYEVENTF_KEYUP, 0);
  keybd_event(VK_CONTROL, MapVirtualKey(VK_CONTROL, 0), KEYEVENTF_KEYUP, 0);
end;

procedure TEventLogDemoMainForm.tvEventTypesClick(Sender: TObject);
begin
  if tvEventTypes.Selected = nil then Exit;
  try
    if tvEventTypes.Selected.ImageIndex < 4 then
    begin
      if not (osRead in EventLog.OpenState) then
      begin
        CurrData := 0;
        acOpenExecute(nil);
        Exit;
      end;
      if CurrData = tvEventTypes.Selected.ImageIndex then Exit;
      CurrData := tvEventTypes.Selected.ImageIndex;
      acOpenExecute(nil);
    end
    else
      if not BackupShowing then
      begin
        lvEvents.Items.BeginUpdate;
        try
          lvEvents.Items.Clear;
          dlgProgress := TdlgProgress.Create(nil);
          try
            dlgProgress.LoadBackupLog(BackupFilePath);
            CurrData := 4;
          finally
            dlgProgress.Release;
            BackupShowing := True;
          end;
        finally
          lvEvents.Items.EndUpdate;
        end;
      end;
  finally
    tvEventTypes.SetFocus;
  end;
end;

procedure TEventLogDemoMainForm.tvEventTypesCollapsing(Sender: TObject;
  Node: TTreeNode; var AllowCollapse: Boolean);
begin
  AllowCollapse := False;
end;

procedure TEventLogDemoMainForm.VerifyRegistration;
begin
  Registered := EventLog.IsRegistered;
  if Registered then
    StatusBar.Panels.Items[1].Text := '    '
  else
    StatusBar.Panels.Items[1].Text := '     ';
end;

end.
